package RPM::Source::Tools::BuildReqStorage;

use Carp;
use strict;
use warnings;

our $VERSION=0.07;
# there is hardcoded rpm limit, about 5000. but more than 800 is not practical.
my $RPM_BR_STRING_LIMIT=800;

sub new {
    my $class = shift;
    my $self = {
	-entry_comment=> 'SourceDeps', # TODO: timestamp?
	-entry_type=>'oneline',
	-post_header_comment => undef,
	-pre_footer_comment => undef,
	-buildreq=>{},
	-buildprereq=>{},
	-ignore_buildreq=>{},
	@_
    };
    bless $self, $class;

    return $self;
}

sub _new_from_br_br_pre {
# TODO
}

sub ignore_buildreq {
    my ($self, @names)=@_;
    foreach my $name (@names) {
	$self->{-ignore_buildreq}->{$name}=1;
    }
}

sub add_buildreq {
    my ($self, @names)=@_;
    my $deps=$self->{-buildreq};
    foreach my $name (@names) {
	$deps->{$name}||='';
    }
}

sub add_buildreq_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-buildreq};
    foreach my $name (@names) {
	my $oldreason=$deps->{$name};
	if ($oldreason) {
	    $oldreason.='; '.$reason;
	} else {
	    $oldreason=$reason;
	}
	$deps->{$name}=$oldreason;
    }
}

sub add_pre_buildreq {
    my ($self, @names)=@_;
    my $deps=$self->{-buildreq};
    my $predeps=$self->{-buildprereq};
    foreach my $name (@names) {
	$deps->{$name}||='';
	$predeps->{$name}||='';
    }
}

sub add_pre_buildreq_for {
    my ($self, $reason, @names)=@_;
    my $deps=$self->{-buildreq};
    my $predeps=$self->{-buildprereq};
    foreach my $name (@names) {
	$deps->{$name}||='';
	my $oldreason=$predeps->{$name};
	if ($oldreason) {
	    $oldreason.='; '.$reason;
	} else {
	    $oldreason=$reason;
	}
	$predeps->{$name}=$oldreason;
    }
}

sub has_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-buildreq};
    return defined $deps->{$name};
}

sub has_pre_buildreq {
    my ($self,$name)=@_;
    my $predeps=$self->{-buildprereq};
    return defined $predeps->{$name};
}

sub get_list_buildreq {
    my ($self,$name)=@_;
    return keys(%{$self->{-buildreq}});
}

sub get_list_pre_buildreq  {
    my ($self,$name)=@_;
    return keys(%{$self->{-buildprereq}});
}

sub del_buildreq {
    my ($self,$name)=@_;
    my $deps=$self->{-buildreq};
    delete $deps->{$name};
}

sub del_pre_buildreq {
    my ($self,$name)=@_;
    my $predeps=$self->{-buildprereq};
    delete $predeps->{$name};
}

# used in buildreq_src only: starting point to refactor;
sub get_entry {
    my ($self,$existing,$existing_pre)=@_;
    my $output='';
    my (@predeps,@deps);
    my $ignore=$self->{-ignore_buildreq};
    my $deps=$self->{-buildreq};
    my $predeps=$self->{-buildprereq};
    # beware of '' reasons!
    foreach my $predep (keys(%$predeps)) {
	push @predeps, $predep if not defined $ignore->{$predep} and not defined $existing_pre->{$predep};
    }
    foreach my $dep (keys(%$deps)) {
	push @deps, $dep if not defined $ignore->{$dep} and not defined $predeps->{$dep} and not defined $existing->{$dep};
    }
    if ($self->{-entry_type} eq 'oneline') {
	map {$output.= 'BuildRequires(pre): '.$_."\n"} &__limited_join($RPM_BR_STRING_LIMIT,' ',sort {$a cmp $b} @predeps);
	map {$output.= 'BuildRequires: '.$_."\n"} &__limited_join($RPM_BR_STRING_LIMIT,' ',sort {$a cmp $b} @deps);
    } elsif ($self->{-entry_type} eq 'multiline') {
	foreach my $breqpre (sort {$a cmp $b} @predeps) {
	    next unless $breqpre;
	    my $reason=$predeps->{$breqpre};
	    $output.= '# '.$reason."\n" if $reason;
	    $output.= 'BuildRequires(pre): '.$breqpre."\n";
	}
	foreach my $breq (sort {$a cmp $b} @deps) {
	    next unless $breq;
	    my $reason=$deps->{$breq};
	    $output.= '# '.$reason."\n" if $reason;
	    $output.= 'BuildRequires: '.$breq."\n";
	}
    } else {
	die "unknown/unsupported entry type ".$self->{-entry_type};
    }
    if ($output) {
	my $post_header_comment=$self->{-post_header_comment};
	$output=$post_header_comment.$output if $post_header_comment;
	my $pre_footer_comment=$self->{-pre_footer_comment};
	$output.=$pre_footer_comment if $pre_footer_comment;
    }
    $output=$self->__entry_header()."\n".$output.$self->__entry_footer()."\n" if $output;
    return $output;
}

sub get_spec_entry {
    my ($self,$spec,$is_keep_old_entry)=@_;
    my ($existing,$existing_pre)=$self->current_buildreq($spec,$is_keep_old_entry);
    return $self->get_entry($existing,$existing_pre);
}

sub update_spec_entry {
    my ($self,$spec,$is_keep_old_entry)=@_;
    my $mainsec=$spec->main_section;
    my @main_body_copy=@{$mainsec->get_bodyref};
    my $update_failed;
    my $is_old_entry_found=0;
    my $begin_anchor=$self->__entry_header_anchor;
    my $end_anchor=$self->__entry_footer_anchor;
    my ($old,$old_pre)=({},{});
    $mainsec->map_body (sub {
	return if $is_old_entry_found>1;
	if (/$begin_anchor/) {
	    $is_old_entry_found=1; $_=''; return;
	} elsif (not $is_old_entry_found) {
	    return;
	}
	if (/^BuildRequires/) {
	    #print "BR:$_";
	    $_=''; return;
	} elsif (/$end_anchor/) {
	    $is_old_entry_found=2;
	    my ($existing,$existing_pre)=$self->current_buildreq($spec,$is_keep_old_entry);
   	    $_=$self->get_entry($existing,$existing_pre);
	    return;
	} elsif (/^#/) { # other comment
	    $_=''; return;
	} elsif (/^\%(?:if|endif)/) { # RPM conditoinals are allowed too
	    $_=''; return;
	} else {
	    $update_failed=1;
	}
			});

    if ($is_old_entry_found ==1 or $update_failed) {
	warn 'BuildReqStorage: old spec entry '.$self->__entry_header()." is broken, update failed\n";
	$mainsec->set_body(\@main_body_copy);
	$is_old_entry_found=0;
    }
    $mainsec->unshift_body($self->get_spec_entry($spec)) if not $is_old_entry_found;
}

#------------------------------------------------------------------
# protected; used in buildreq-src and DistroMap::SourceAnalyzer
#------------------------------------------------------------------

sub current_buildreq {
    my ($self,$spec,$is_merge_old_entry)=@_;
    my ($out,$outpre)=({},{});

    my $is_old_entry_found=0;
    my $begin_anchor=$self->__entry_header_anchor;
    my $end_anchor=$self->__entry_footer_anchor;
    my ($old,$old_pre)=({},{});
    if ($spec) {
	foreach my $section ($spec->get_sections()) {
	    next if $section->get_type() ne 'package';
	    if (not $section->is_main_section) {
		foreach (@{$section->get_bodyref()}) {
		    &__add_dep($_,$out,$outpre);
		}
	    } else {
		$section->map_body (sub {
		    if ($is_old_entry_found > 1) {
			&__add_dep($_,$out,$outpre);
		    } elsif (/$begin_anchor/) {
			$is_old_entry_found=1; 
		    } elsif (not $is_old_entry_found) {
			&__add_dep($_,$out,$outpre);
		    } elsif (/$end_anchor/) {
			$is_old_entry_found=2;
		    } else { # (/^BuildRequires/) { 
			# we are inside an entry; Let us support even a broken entry.
			#print "BR:$_";
			&__add_dep($_,$old,$old_pre);
		    }
				    });
		
	    }
	}
	if ($is_merge_old_entry) {
	    map {$outpre->{$_}=1 } keys (%$old_pre);
	    map {$out->{$_}=1 } keys (%$old);
	}
    }
    
    return wantarray ? ($out,$outpre,$old,$old_pre) : $out;
}

#------------------------------------------------------------------
# private methods
#------------------------------------------------------------------

sub __entry_header_anchor {
    my ($self)=@_;
    return '^# BEGIN '."\Q$self->{-entry_comment}\E".'\('.$self->{-entry_type}.'\):';
}

sub __entry_footer_anchor {
    my ($self)=@_;
    return '^# END';
}

sub __entry_header {
    my ($self)=@_;
    return '# BEGIN '.$self->{-entry_comment}.'('.$self->{-entry_type}.'):';
}

sub __entry_footer {
    my ($self)=@_;
    return '# END '.$self->{-entry_comment}.'('.$self->{-entry_type}.')';
}

#------------------------------------------------------------------
# private static
#------------------------------------------------------------------

sub __limited_join {
    my ($limit,$sep,@array)=@_;
    my @output;
    return @output unless @array;
    my $joined=shift @array;
    while (@array) {
	if (length($joined) > $limit) {
	    push @output, $joined;
	    $joined=shift @array;
	} else {
	    $joined.=$sep.shift(@array);
	}
    }
    push @output, $joined;
    return @output;
}

sub __add_dep {
    my ($line,$out,$outpre)=@_;
    if ($line=~/^\s*(\%{\?!?[^:]+:)?\s*((?:BuildReq|BuildRequires|BuildPreReq|BuildPrereqs)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/i) {
	my $macro=$1;
	my $Tag=$2;
	my $space=$3;
	my $reqlist=$4;
	my $endmacro='';
	if ($macro) {
	    $reqlist=~s/\s*(\})\s*$//;
	    $endmacro=$1;
	}
	my $names = &__get_pkg_names($reqlist);
	if ($Tag=~/pre/i) {
	    map {$out->{$_}=1} @$names;
	    map {$outpre->{$_}=1} @$names;
	} else {
	    map {$out->{$_}=1} @$names;
	}
    }
}

sub __get_pkg_names {
    my ($line)=@_;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    # TODO: altlinux specific
    #$line =~ s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $line;
    my @out;
    while (@in) {
	my $token = shift @in;
	if ($token =~/^[<=>]+$/) {
	    shift @in;
	} else {
	    push @out, $token;
	}
    }
    return \@out;
};

1;
