package RPM::Source::Tools::Dependency::Filter;

use strict;
use warnings;
use constant {
	SPEC_TAG_BUILD_TAG_MASK	=> (1 << 0) | (1 << 1),

	SPEC_TAG_BUILD_REQUIRES	=> (1 << 0),
	SPEC_TAG_BUILD_CONFLICTS => (1 << 1),
	SPEC_TAG_REQUIRES	=> (1 << 2),
	SPEC_TAG_PROVIDES	=> (1 << 3),
	SPEC_TAG_CONFLICTS	=> (1 << 4),
	SPEC_TAG_OBSOLETES	=> (1 << 5),

	SPEC_TAG_UNKNOWN	=> (1 << 6),

	SPEC_FLAG_PREREQ     => (1 << 6),     #/*!< @todo Legacy. */
	SPEC_FLAG_SCRIPT_PRE => ((1 << 9)|(1 << 6)), #/*!< %pre dependency. */
	SPEC_FLAG_SCRIPT_POST => ((1 << 10)|(1 << 6)), #/*!< %post dependency. */
	SPEC_FLAG_SCRIPT_PREUN => ((1 << 11)|(1 << 6)), #/*!< %preun dependency. */
	SPEC_FLAG_SCRIPT_POSTUN => ((1 << 12)|(1 << 6)), #/*!< %postun dependency. */
	SPEC_FLAG_SCRIPT_VERIFY => (1 << 13), #/*!< %verify dependency. */
#	SPEC_FLAG_FIND_REQUIRES => (1 << 14), #/*!< find-requires generated dependency. */
#	SPEC_FLAG_FIND_PROVIDES => (1 << 15), #/*!< find-provides generated dependency. */
	SPEC_FLAG_TRIGGERIN  => (1 << 16),    #/*!< %triggerin dependency. */
	SPEC_FLAG_TRIGGERUN  => (1 << 17),    #/*!< %triggerun dependency. */
	SPEC_FLAG_TRIGGERPOSTUN => (1 << 18), #/*!< %triggerpostun dependency. */
#	SPEC_FLAG_SCRIPT_PREP => (1 << 20),   #/*!< %prep build dependency. */
#	SPEC_FLAG_SCRIPT_BUILD => (1 << 21),  #/*!< %build build dependency. */
#	SPEC_FLAG_SCRIPT_INSTALL => (1 << 22),#/*!< %install build dependency. */
#	SPEC_FLAG_SCRIPT_CLEAN => (1 << 23),  #/*!< %clean build dependency. */
#	SPEC_FLAG_RPMLIB     => ((1 << 24) | SPEC_FLAG_PREREQ), #/*!< rpmlib(feature) dependency. */
#/*@-enummemuse@*/
#	SPEC_FLAG_TRIGGERPREIN => (1 << 25),  #/*!< @todo Implement %triggerprein. */
#/*@=enummemuse@*/
#/*@-enummemuse@*/
#    SPEC_FLAG_KEYRING    => (1 << 26)
#/*@=enummemuse@*/
};

use Exporter 'import';
our @EXPORT = qw/
	SPEC_TAG_BUILD_TAG_MASK
	SPEC_TAG_BUILD_REQUIRES
	SPEC_TAG_BUILD_CONFLICTS
	SPEC_TAG_REQUIRES
	SPEC_TAG_PROVIDES
	SPEC_TAG_CONFLICTS
	SPEC_TAG_OBSOLETES
	SPEC_TAG_UNKNOWN

	SPEC_FLAG_PREREQ
	SPEC_FLAG_SCRIPT_PRE
	SPEC_FLAG_SCRIPT_POST
	SPEC_FLAG_SCRIPT_PREUN
	SPEC_FLAG_SCRIPT_POSTUN
	SPEC_FLAG_SCRIPT_VERIFY
	SPEC_FLAG_TRIGGERIN
	SPEC_FLAG_TRIGGERUN
	SPEC_FLAG_TRIGGERPOSTUN
/;

our $version=0.002;
our $verbose=0;

sub new {
    my $class = shift;
    my $self = {
	-tags => SPEC_TAG_BUILD_REQUIRES | SPEC_TAG_BUILD_CONFLICTS | SPEC_TAG_REQUIRES | SPEC_TAG_CONFLICTS,
#	-flags => 
	@_};
    bless $self, $class;
}

# Optional method, used in DependencyFilterFactory.
# Decided not to move into an inherited class, but declare optional.
sub transformation_factory_init {
    my ($self,$spec,$parent,$opt,$resourceobj)=@_;
    $self->{-spec}=$spec;
    $self->{-parent}=$parent;
    $self->{-opt}=$opt;
    $self->{-resourceobj}=$resourceobj;
}

sub is_applicable {
    my ($self,$tagobj)=@_;
    my $mask=$self->{-tags};
    return 1 unless defined $mask;
    print STDERR ref($self),"->is_applicable: ",$tagobj->tagstring," mask=$mask tag=",$tagobj->tagbits," is_applicable=",$mask & $tagobj->tagbits,"\n" if $verbose>1;
    return $mask & $tagobj->tagbits;
}

sub filter {
    my ($self,$tagobj, $ref)=@_;
    # check target criteria here
    return @$ref if not $self->is_applicable($tagobj);
    my @out;
    foreach my $dep (@$ref) {
	my @filtered_deps=$self->apply($tagobj,$dep);
	foreach my $ret (@filtered_deps) {
	    my $ref=ref($ret);
	    if (not $ref or $ref ne 'RPM::Source::Tools::Dependency') {
		warn "filter ".ref($self).": apply returned invalid object [$ref]";
	    } else {
		push @out, $ret if defined $ret->{-name};
	    }
	}
    }
    return @out;
}

sub apply {
    # in subclasses
    ...
}

package RPM::Source::Tools::Dependency::Tag;

# TODO: object?
sub new {
    my ($class,$tagstring,$flagstring)=@_;
    # TODO
    my ($tagbits,$flagbits)=(0,0);
    $flagbits=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_PREREQ if $tagstring=~/pre/i;
    if ($tagstring=~/prov/i) {
	$tagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_PROVIDES;
    } elsif ($tagstring=~/req/i) {
	$tagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_REQUIRES;
    } elsif ($tagstring=~/obso/i) {
	$tagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_OBSOLETES;
    } elsif ($tagstring=~/confl/i) {
	$tagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_CONFLICTS;
    }
    if ($tagstring=~/build/i) {
	if ($tagbits == RPM::Source::Tools::Dependency::Filter::SPEC_TAG_REQUIRES) {
	    $tagbits=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_BUILD_REQUIRES;
	} elsif ($tagbits == RPM::Source::Tools::Dependency::Filter::SPEC_TAG_CONFLICTS) {
	    $tagbits=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_BUILD_CONFLICTS;
	} else {
	    warn "WARNING: Dependency::Tag: unknown tag $tagstring\n";
	    $tagbits=RPM::Source::Tools::Dependency::Filter::SPEC_TAG_UNKNOWN;
	}
    }
    if ($tagstring=~/\((.+)\)$/i) {
	my @flags=map {lc($_)} split(/\s*,\s*/,$1);
	foreach my $flag (@flags) {
	    if ($flag eq 'pre') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_SCRIPT_PRE;
	    } elsif ($flag eq 'post') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_SCRIPT_POST;
	    } elsif ($flag eq 'preun') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_SCRIPT_PREUN;
	    } elsif ($flag eq 'postun') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_SCRIPT_POSTUN;
	    } elsif ($flag eq 'verify') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_SCRIPT_VERIFY;
#	    } elsif ($flag eq 'triggerprein') {
#		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_TRIGGERPREIN;
	    } elsif ($flag eq 'triggerin') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_TRIGGERIN;
	    } elsif ($flag eq 'triggerun') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_TRIGGERUN;
	    } elsif ($flag eq 'triggerpostun') {
		$flagbits|=RPM::Source::Tools::Dependency::Filter::SPEC_FLAG_TRIGGERPOSTUN;
	    }
	}
    }
    my $self=[$tagbits,$flagbits,$tagstring];
    bless $self,$class;
}

sub tagbits {
    return $_[0]->[0];
}

sub flagbits {
    return $_[0]->[1];
}

sub tagstring {
    return $_[0]->[2];
}

package RPM::Source::Tools::Dependency::Util;

use strict;
use warnings;

our $verbose=0;

sub filter_section {
    my ($section,$filters_ref)=@_;
    my @out;
    foreach (@{$section->get_bodyref()}) {
#%{?!_without_fuse:BuildRequires: fuse-devel}
	if (/^(\s*(\%{!?\?[^:]+:)?\s*(BuildReq|BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts|Provides|Obsoletes)\s*(?:\((\w+)\))?\s*:)(.*\S)(\s*)$/i) {
	    my $Prefix=$1;
	    my $macro=$2;
	    my $tagstring=$3;
	    my $flagstring=$4;
	    my $reqlist=$5;
	    my $endline=$6;
	    my $endmacro='';
	    if ($macro) {
		$reqlist=~s/(\s*\})\s*$//;
		$endmacro=$1;
	    }
	    my $tagobj=RPM::Source::Tools::Dependency::Tag->new($tagstring,$flagstring);
	    my @deplist=&RPM::Source::Tools::Dependency::Util::tokenize_pkg_list($reqlist);
	    foreach my $filter (@$filters_ref) {
		@deplist=$filter->filter($tagobj,\@deplist);
	    }
	    if (@deplist > 0) {
		my $filtered=&RPM::Source::Tools::Dependency::Util::restore_pkg_list(@deplist);
		print STDERR "Filtered -> $Prefix$filtered$endmacro\n" if $verbose;
		push @out, "$Prefix$filtered$endmacro$endline" if ($filtered);
	    } else {
		print STDERR "Dropped: $Prefix$reqlist$endmacro$endline" if $verbose;
	    }
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

sub tokenize_pkg_list {
    my ($line)=@_;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    # TODO: altlinux specific
    # $line =~ s/\%\{dist\}//g;
    chomp $line;
    my @out;
  TOKEN:
    while (length($line)) {
	$line=~s/^([\s,]*)//;
	my $padspace=$1;
	$line=~s/^([^\s,]*)//;
	my $name=$1;
	next if $name eq '';
	# pre-support for %(eval ....)
	if ($name =~/^\%\(/) {
	    while ($name !~/\)$/ and length($line)) {
		$line=~s/^(\s*\S+)//;
		$name .= $1;
	    }
	# support for baa%{?foo: bla bla bla... }
	} elsif ($name =~/\%\{[^\}]+$/) {
	    while ($name !~/\}$/ and length($line)) {
		$line=~s/^(\s*\S+)//;
		$name .= $1;
	    }
	}
	my @dep=(
	    -padspace=>$padspace,
	    -name => $name,
	    );
	if ($line=~s/^(\s*)([<=>]+)(\s*)([^\s,]*)//) {
	    push @dep, -lspace => $1, -relation => $2, -rspace => $3, -version => $4;
	}
	push @out,RPM::Source::Tools::Dependency->new(@dep);
    }
    return @out;
};

sub restore_pkg_list {
    my $line=join('',map {$_->to_string} @_);
}

package RPM::Source::Tools::Dependency;

use strict;
use warnings;

sub new {
    my $class = shift;
    my $self = {@_};
    for my $i (qw/-padspace -lspace -rspace/) {
	$self->{$i}=' ' if defined $self->{$i} and $self->{$i} eq '';
    }
    bless $self, $class;
}

sub to_string {
    my $self=shift;
    return '' if not defined $self->{-name};
    for my $i (qw/-padspace -lspace -rspace/) {
	$self->{$i}=' ' if not defined $self->{$i};
    }
    my $string=$self->{-padspace}.$self->{-name};
    if (defined $self->{-version}) {
	$string.=$self->{-lspace}.$self->{-relation}.$self->{-rspace}.$self->{-version};
    }
    return $string;
}

1;
