package RPM::Source::Tools::PackageFilter;

use Carp;
use strict;
use warnings;

sub new {
    my $class = shift;
    my $self = {@_};
    bless $self, $class;
}

sub init {
    my ($self, $converter, $spec)=@_;
    $self->{CONVERTER}=$converter;
    $self->{SPEC}=$spec;
}

# returns:
# if filter does not apply -> false
# if filter does apply ->
#   the ref to the list of tokens pkgname is mapped
#   an empty list means pkgname should be dropped
sub map_pkg {
    my ($self, $pkgname)=@_;
    ...
}

# public static 
sub filter_pkg_list {
    my ($line, $SKIP, $SUBST, $DROP_VER, $FILTERS)=@_;
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    $line =~ s/\>\s+=/>=/g;
    $line =~ s/\<\s+=/<=/g;
    # TODO: altlinux specific
    $line =~ s/\%\{dist\}//g;

    my @in = split /[\s,]+/, $line;
    my @out;
  TOKEN:
    while (@in) {
	my $token = shift @in;
	if ($token =~/^[<=>]+$/) {
	    push @out, $token;
	    push @out, shift @in;
	} elsif ($SKIP->{$token}) {
	    &read_tag_version(\@in);
	} elsif ($SUBST->{$token}) {
	    push @out, $SUBST->{$token};
	    &read_tag_version(\@in) if $DROP_VER->{$token};
	} else {
	    foreach my $filter (@$FILTERS) {
		my $mapping=$filter->map_pkg($token);
		if ($mapping) {
		    die 'internal error: not an array ref $mapping in filter '.ref($filter)."\n" unless ref $mapping eq 'ARRAY';
		    if (@$mapping) {
			push @out, join(' ',@$mapping);
			&read_tag_version(\@in) if $DROP_VER->{$mapping->[0]};
		    } else {
			&read_tag_version(\@in);
		    }
		    next TOKEN;
		}
	    }
	    push @out, $token;
	    &read_tag_version(\@in) if $DROP_VER->{$token};
	}
    }
    return join(' ',@out);
};


sub read_tag_version {
    my $ptr=shift;
    return if ($#{$ptr}<1);
    if ($ptr->[0] =~/^(?:<|>|=|!=|>=|<=)$/) {
	return (shift @$ptr, shift @$ptr);
    }
    return;
}

1;
