package RPM::Source::Transformation::Factory::DependencyFilter;
use strict;
use warnings;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Abstract::PriorityCollection;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# group filters with the same transformation priority into the single transformation.
our $group_filters_by_transformation=1;
our $verbose=0;

my @FILTER_ARGS_STORAGE;
my %FILTER_BY_PRIORITY;

use Exporter 'import';
our @EXPORT = qw(
	FILTER_PRIORITY_AT_BAGIN
	FILTER_PRIORITY_PREDISTROMAP
	FILTER_PRIORITY_DISTROMAP
	FILTER_PRIORITY_POSTDISTROMAP
	FILTER_PRIORITY_DEFAULT
	FILTER_PRIORITY_POSTFILTER
	FILTER_PRIORITY_AT_END
);

use constant {
	FILTER_PRIORITY_AT_BAGIN	=> 90,
	FILTER_PRIORITY_PREDISTROMAP	=> 70,
	FILTER_PRIORITY_DISTROMAP	=> 60,
	FILTER_PRIORITY_POSTDISTROMAP	=> 50,
	FILTER_PRIORITY_DEFAULT		=> 0,
	FILTER_PRIORITY_POSTFILTER	=> -10,
	FILTER_PRIORITY_AT_END		=> -20,
};

sub register_filter {
    my $args={@_};
    die "filter class is mandatory" unless $args->{-class};
    $args->{-priority}=FILTER_PRIORITY_DEFAULT if not defined $args->{-priority};
    $args->{-transformation_priority}=PRIORITY_CONVERT if not defined $args->{-transformation_priority};
    push @FILTER_ARGS_STORAGE, $args;
}

sub init_factory {
    my (undef,$resourceobj)=@_;
    foreach my $args (@FILTER_ARGS_STORAGE) {
	my $transformation_priority=$args->{-transformation_priority};
	my $filter=$args->{-class}->new(%$args,
					-resourceobj=>$resourceobj);
	my $by_transformation_priority=$FILTER_BY_PRIORITY{$transformation_priority};
	if (not $by_transformation_priority) {
	    $by_transformation_priority=RPM::Source::Abstract::PriorityCollection->new();
	    $FILTER_BY_PRIORITY{$transformation_priority}=$by_transformation_priority;
	}
	$by_transformation_priority->add($filter);
    }
}

sub transformations {
    my @ret;
    foreach my $transformation_priority (sort keys(%FILTER_BY_PRIORITY)) {
	if ($group_filters_by_transformation) {
	    my @filters;
	    my $sname='Dependency Filters: ';
	    my @names;
	    my @descriptions;
	    foreach my $filter ($FILTER_BY_PRIORITY{$transformation_priority}->get()) {
		push @filters, $filter;
		my $filter_name=$filter->{-name};
		$filter_name=$filter->{-class} if not defined $filter_name;
		push @names, $filter_name;
		push @descriptions, 'Description for the dependency filter '.$filter_name."\n".$filter->{-description} if defined $filter->{-description};
	    }
	    my @wrapper_args=(
		-filters => \@filters,
		-priority => $transformation_priority,
		-opt_handler_sub=>\&opt_handler_dependency_filter
		);
	    push @wrapper_args, -name => 'Dependency Filters: '.join(',',@names) if @names;
	    push @wrapper_args, -description => join(',',@descriptions) if @descriptions;
	    push @ret, RPM::Source::Transformation::OptHandlerWrapper->new(@wrapper_args);
	} else {
	    foreach my $filter ($FILTER_BY_PRIORITY{$transformation_priority}->get()) {
		my $filter_name=$filter->{-name};
		$filter_name=$filter->{-class} if not defined $filter_name;
		my @wrapper_args=(
		    -filters => [$filter],
		    -name => 'Dependency Filter: '.$filter_name,
		    -priority => $transformation_priority,
		    -opt_handler_sub=>\&opt_handler_dependency_filter
		    );
		push @wrapper_args, -description => 'Dependency Filter: '.$filter->{-description} if defined $filter->{-description};
		push @ret, RPM::Source::Transformation::OptHandlerWrapper->new(@wrapper_args);
	    }
	}
    }
    return @ret;
}

sub opt_handler_dependency_filter {
    my ($opt_handler,$spec,$parent,$opt,$resourceobj)=@_;
    my $CLASSES=$opt_handler->{-filters};
    foreach my $filter (@$CLASSES) {
	$filter->transformation_factory_init($spec,$parent,$opt,$resourceobj);
    }
    foreach my $sec ($spec->get_sections()) {
	&RPM::Source::Tools::Dependency::Util::filter_section($sec,$CLASSES) if $sec->get_type() eq 'package';
    }
    return RPM::Source::Transformation::Factory::TRANSFORMATION_OK;
}

1;
