package RPM::Source::Transformation::PkgWriter;

use strict;
use warnings;

use RPM::Source::Transformation::Component;
our @ISA=qw/RPM::Source::Transformation::Component/;

push @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;
$RPM::Source::TransformContainer::WRITER=__PACKAGE__;

# opt'ified, seems to be not used as defaults
our (
     $opt_inplace, $opt_output_hasher_tar,
     # TODO: make them $opt{}'ed and documented in help
     $opt_write_spec,
);
my $opt_transaction_output_subdir;

my $fallback_hasher_tar=1;
# TODO: make it $opt{}'ed and documented in help
my $opt_build_srpm=1;

#### and used somewhere
our $outputdir='.';
our $verbose=1;

sub longopt_usage {
    print "
  Output options:
	--outdir /path/to/output_dir (default is $outputdir)
	-i|--inplace	Edit existing spec file (use together with --spec)
	--hashertar	Write hasher-ready tar file instead of src.rpm file.
";
#	--transaction-output-subdir 'subpath' 
}

our @LONGOPT=(
    'out|outdir=s' => \$outputdir,
    'hashertar'	=> \$opt_output_hasher_tar,
    'i|inplace'	=> \$opt_inplace,
    'build-srpm!'	=> \$opt_build_srpm,
    'w|writespec=s'	=> \$opt_write_spec,
    'transaction-output-subdir=s'	=> \$opt_transaction_output_subdir,
);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-inplace => $opt_inplace,
	-output_hasher_tar => $opt_output_hasher_tar,
	-outputdir => $outputdir,
	-transaction_output_subdir => $opt_transaction_output_subdir,
    });
}

sub write {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $write_rpm_ok=1;
    print STDERR $spec->get_spec() if $verbose>4;
    my $macros=$spec->macros();
    # TODO: internal access!!! should be the write_rpm options!
    $spec->{OUTPUTDIR}=$macros->macro_subst($opt->{-outputdir}) if $opt->{-outputdir};
    $spec->{OUTPUTDIR}=$macros->macro_subst($opt->{-outputdir}.'/'.$opt->{-transaction_output_subdir}) if $opt->{-transaction_output_subdir};
    

    $spec->{-fallback_hasher_tar} = $fallback_hasher_tar;
    if ($opt->{-inplace}) {
	$spec->write_spec();
    } elsif ($opt_write_spec) {
	$spec->write_spec($macros->macro_subst($opt_write_spec));
    } else {
	if ($opt_build_srpm) {
	    if ($opt->{-output_hasher_tar}) {
		$spec->write_hasher_tar();
	    } else {
		$write_rpm_ok=$spec->write_rpm();
		#print STDERR "Transformation::PkgWriter: write_rpm() failed.\n" if $verbose and not $write_rpm_ok;
	    }
	}
    }
    return $write_rpm_ok;
}

1;
