package RPM::Source::Transformation::Resource::Datapath;

use strict;
use warnings;
use Carp;
use File::Basename;
use Cwd;

use RPM::Source::Transformation::Resource;
our @ISA=qw/RPM::Source::Transformation::Resource/;

push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

# slave factory to set -copypath in spec
use RPM::Source::Transformation::Factory::Datapath;

our @datapath=(cwd());
push @datapath, dirname($0) if $0 =~ m'^/home/';
push @datapath, '/usr/share/srpmtools';
our @patchdir;

### datapath hierarchy ###
#./hooks INC. look for default hook
#./common INC (TODO: lib ?)
#./patches (copy from)
#TODO:./sources (copy from)
#./groups

# TODO: opt
my $opt_policy_hook_name; # accept values: undef, old, new, both
my (@opt_datapath);

our $verbose=0;

sub longopt_usage {
    print "
  Filesystem resources options:
	--datapath /path1:/path2:... colon separated list of directories to search for hooks/, patches/, etc...
";
}

our @LONGOPT=(
    'datapath=s'	=> \@opt_datapath,
);

sub process_options {
    my ($self)=@_;
    @datapath=map {split(':',$_)} @opt_datapath if @opt_datapath;
    # for cross-references in hooks
    push @INC, grep {-d $_} map {$_."/hooks", $_."/common", $_."/lib"} @datapath;
    @patchdir=grep {-d $_} map {$_."/patches"} @datapath;
}

sub prepare_resource {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    my $name = $spec->main_section()->get_tag('Name');
    my $oldname = $name;
    my $_rename;
    my $macros=$spec->macros();
    $_rename=$opt->{-rename} if $opt->{-rename} and $opt->{-rename} ne 'none';
    $_rename=$macros->macro_subst($_rename) if $_rename;
    $name=$_rename if defined $_rename;
    $resourceobj->{-_name_}=$name;
    $resourceobj->{-_oldname_}=$oldname;
}



# private datapath methods...

sub __find_associated_file {
    my ($pathref,$prefix,$suffix,$name,$oldname)=@_;
    Carp::confess "prefix and suffix are required" unless defined $prefix and defined $suffix;
    my @hook_names=($name);
    if (defined $oldname and $name ne $oldname) {
	if (not $opt_policy_hook_name or $opt_policy_hook_name eq 'both') {
	    @hook_names=($name,$oldname);
	} elsif ($opt_policy_hook_name eq 'old') {
	    @hook_names=($oldname);
	} elsif ($opt_policy_hook_name eq 'new') {
	    @hook_names=($name);
	} else {
	    die "invalid argument $opt_policy_hook_name for hook name policy";
	}
    }
    print STDERR "DEBUG: DatapathResource: path=",join(':',@$pathref)," prefix=$prefix suffix=$suffix names=",join(',',@hook_names),"\n" if $verbose>1;
    return &__find_first_in_datapath($pathref,map {"$prefix/$_$suffix"} @hook_names);
}

sub __find_first_in_datapath {
    my ($pathref,@files)=@_;
    my @found;
  HOOK_SEARCH:
    foreach my $fname (@files) {
	print STDERR "looking for $fname\n" if $verbose>1;
	foreach my $file (grep {-e $_} map {"$_/$fname"} @$pathref) {
	    print STDERR "found $file\n" if $verbose;
	    push @found, $file;
	    last HOOK_SEARCH; # default hook should be only one
	}
    }
    return @found;
}


package RPM::Source::Shared::ResourceContainer;

sub find_named_filesystem_resources {
    my ($resourceobj,$prefix,$suffix)=@_;
    my $name=$resourceobj->{-_name_};
    my $oldname=$resourceobj->{-_oldname_};
    my $datapathref=$resourceobj->__datapathref;
    my @resources=&RPM::Source::Transformation::Resource::Datapath::__find_associated_file($datapathref,$prefix,$suffix,$name,$oldname);
    return @resources;
}

# oops! RPM::Source::Transformation::Resource::Datapath::patchdir is used in get_spec :(
# used in CommandLineFactory
sub patchdirref {
    my ($self)=shift;
    return \@RPM::Source::Transformation::Resource::Datapath::patchdir;
}

# not used yet except above in find_named_filesystem_resources
sub __datapathref {
    my ($self)=shift;
    return \@RPM::Source::Transformation::Resource::Datapath::datapath;
}


1;
